## Branches

Branch-off within the same repository in order to stay safe!

<br><br>The master branch
<img src="img/branch-master.png" class="branch-master" />


## One branch per feature

Assume that you want to work on a function for a matrix-vector operation.
```bash
$ git checkout -b matrix_vect_mult_myName
# creates the branch locally
```
The `-b` flag creates the branch.

<img src="img/branch-create.png" class="branch-create" />

Push the branch to the remote repository
```bash
$ git push
```


If you do that, `git` might complain
```bash
fatal: The current branch matrix_vect_mult_myName has no upstream branch.
To push the current branch and set the remote as upstream, use

    git push --set-upstream origin matrix_vect_mult_myName
```

<br>
Follow the advice and do
```bash
$ git push --set-upstream origin matrix_vect_mult_myName
```


## Switch between branches

In your terminal, you may see the name of the branch you are on.

List available branches of the repository
```bash
$ git branch --list
```

<div class="fragment">
<br>
Checkout another branch
```bash
$ git checkout <branch_name>
```

<div class="fragment">
<br>
You can switch back to the `master` branch with
```bash
$ git checkout master
```


<br>
You can use the 5 essential commands as before.

Only difference: you are on your own branch.

<img src="img/branch-commit.png" class="branch-commit" />


## Merge a branch

If you want your feature on the `develop` or `master` branches,
**submit a MR or a PR** via the Github/Gitlab interface.

<br>
Use the **interface** to make use of your peers to review your code!

<img src="img/branch-merge.png" class="branch-merge" />

<br>
Once merged, you can delete the branch via the interface.


## Gitlab interface
<img src="https://gitlab.com/gitlab-com/gitlab-artwork/raw/master/logo/logo.png" alt="GitLab" style="width: 100px;"/>

Detailed information is on [docs.gitlab.com/ce/gitlab-basics/add-merge-request.html](https://docs.gitlab.com/ce/gitlab-basics/add-merge-request.html).

1. Click on **New merge request**

![New Merge Request](https://docs.gitlab.com/ee/gitlab-basics/img/merge_request_new.png)
<br><br>
2. Compare the branches

![Compare branches](https://docs.gitlab.com/ee/gitlab-basics/img/merge_request_select_branch.png)


<ol start="3">
  <li>Assign your peer
  <br>

  ![Assigning a peer](https://gitlab.com/gitlab-org/gitlab-ce/uploads/6445f80f96d8cb85b320322d36d8bb0e/assigneesidebar.png)
  </li>
</ol>


<ol start="4">
  <li>Submit the MR

  ![Submit MR](https://docs.gitlab.com/ee/gitlab-basics/img/merge_request_page.png)</li>
</ol>


## Github interface
<img src="https://assets-cdn.github.com/images/modules/logos_page/GitHub-Mark.png" alt="GitHub" style="width: 100px;"/>

Detailed information is on [help.github.com/articles/creating-a-pull-request/](https://help.github.com/articles/creating-a-pull-request/).

1. Click on **New pull request**

![New pull request](https://help.github.com/assets/images/help/pull_requests/pull-request-start-review-button.png)
<br><br>
2. Compare the branches

![Compare branches](https://help.github.com/assets/images/help/pull_requests/choose-base-and-compare-branches.png)


<ol start="3">
  <li>Assign your peer
  <br>
  
  ![Assigning a peer](https://help.github.com/assets/images/help/issues/issues_assigning_dropdown.png)
  </li>
</ol>


<ol start="4">
  <li>Submit the PR

  ![Submit MR](https://help.github.com/assets/images/help/pull_requests/pullrequest-send.png)</li>
</ol>
